﻿Module Module_SaveLoad

    Friend CustomColors(-1) As Int32
    Friend EventsAreEnabled As Boolean = False

    ' ==============================================================================================================
    '  Operating System and Platform data
    ' ==============================================================================================================
    Friend OperatingSystemIsWindows As Boolean = True
    Friend OperatingSystemIs_XP_or_Vista As Boolean = False

    Friend Sub InitPlatformData()
        Select Case Environment.OSVersion.Platform
            Case PlatformID.Win32NT, PlatformID.Win32S, PlatformID.Win32Windows, PlatformID.WinCE
            Case Else
                OperatingSystemIsWindows = False
        End Select
        If OperatingSystemIsWindows AndAlso Environment.OSVersion.Version.Major <= 6 Then
            OperatingSystemIs_XP_or_Vista = True
        End If
    End Sub

    ' =======================================================================================================
    '   Platform file names corrections
    ' =======================================================================================================
    Friend Function PlatformAdjustedFileName(ByVal FileName As String, _
                                             Optional ByVal DefaultName As String = "") As String
        If OperatingSystemIsWindows Then
            FileName = Replace(FileName, "/", "\")
            Return FileName
        Else
            If FileName.Contains(":") Then FileName = DefaultName
            FileName = Replace(FileName, "\", "/")
            Return FileName
        End If
    End Function

    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' =====================================================================================
    '  Helper functions  
    ' =====================================================================================
    'Public Function StrToFloat(ByVal s As String) As Double
    '    StrToFloat = Val(Replace(s, ",", "."))
    'End Function
    Public Function FloatToStr(ByVal n As Double) As String
        FloatToStr = Trim(Str(n))
        If Strings.Left(FloatToStr, 1) = "." Then FloatToStr = "0" & FloatToStr
    End Function
    Public Function FloatToStr_3(ByVal n As Double) As String
        FloatToStr_3 = Format(n, "0.000")
        FloatToStr_3 = Replace(FloatToStr_3, ",", ".")
    End Function
    Public Function BoolToStr(ByVal b As Boolean) As String
        If b Then
            BoolToStr = "1"
        Else
            BoolToStr = "0"
        End If
    End Function
    Public Function ExtractBoolean(ByRef s As String) As Boolean
        ExtractBoolean = Val(s) <> 0
    End Function
    Public Function ExtractInteger(ByRef s As String, ByVal min As Integer, ByVal max As Integer) As Int32
        ExtractInteger = Val_Int(s)
        If ExtractInteger < min Then ExtractInteger = min
        If ExtractInteger > max Then ExtractInteger = max
    End Function
    Public Function ExtractDouble(ByRef s As String, ByVal min As Double, ByVal max As Double) As Double
        ExtractDouble = Val(s)
        If ExtractDouble < min Then ExtractDouble = min
        If ExtractDouble > max Then ExtractDouble = max
    End Function
    Public Function ExtractSingle(ByRef s As String, ByVal min As Single, ByVal max As Single) As Single
        ExtractSingle = CSng(Val(s))
        If ExtractSingle < min Then ExtractSingle = min
        If ExtractSingle > max Then ExtractSingle = max
    End Function
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function
    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function
    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function
    Friend Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function
    Friend Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function
    Friend Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function
    Friend Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function
    Friend Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function

    ' =====================================================================================
    '  Colors 
    ' =====================================================================================
    Private Function Color_ToHtmlString(ByVal col As Color) As String
        Return Drawing.ColorTranslator.ToHtml(col)
    End Function
    Private Function Color_FromHtmlString(ByVal HtmlColorString As String) As Color
        On Error Resume Next
        Return Drawing.ColorTranslator.FromHtml(HtmlColorString)
    End Function
    Private Function Color_IntToHtmlString(ByVal Win32Col As Int32) As String
        Dim col As Color = Drawing.ColorTranslator.FromWin32(Win32Col)
        Return Drawing.ColorTranslator.ToHtml(col)
    End Function
    Private Function Color_IntFromHtmlString(ByVal HtmlColorString As String) As Int32
        On Error Resume Next
        Dim col As Color = Drawing.ColorTranslator.FromHtml(HtmlColorString)
        Return Drawing.ColorTranslator.ToWin32(col)
    End Function

    ' =====================================================================================
    '  Files  
    ' =====================================================================================
    Public Sub File_KillFile(ByRef FileName As String)
        On Error Resume Next
        If Not My.Computer.FileSystem.FileExists(FileName) Then Exit Sub
        SetAttr(FileName, FileAttribute.Normal)
        Kill(FileName)
    End Sub


    ' =====================================================================================
    '  Combo functions  
    ' =====================================================================================
    Friend Sub Combo_SetIndex_FromString(ByVal combo As ComboBox, ByVal str As String)
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If str = Nothing Then str = ""
        With combo
            For i As Int32 = 0 To .Items.Count - 1
                If .Items(i).ToString = str Then
                    .SelectedIndex = i
                    Exit For
                End If
            Next
        End With
        EventsAreEnabled = old
    End Sub



    ' ==================================================================================================
    '  SAVE LOAD -- USER OPTIONS
    ' ==================================================================================================
    Friend Sub SaveUserOptions()
        '
        Dim iniFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & _
                                                             AssemblyName() & "_UserOptions.txt")
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            f.WriteLine(" Form main")
            f.WriteLine("======================================================================")
            ' ------------------------------------------------------------------------------ FORM MAIN BOUNDS
            ' (The value of the RestoreBounds property is valid only 
            '   when the WindowState property of the Form class is not equal to Normal)
            Dim r As Rectangle
            r = GetFormRectangle(Form_Main)
            'Dim a As Int32 = 0 : a = a \ a
            f.WriteLine(TabString("Top", r.Top))
            f.WriteLine(TabString("Left", r.Left))
            f.WriteLine(TabString("Width", r.Width))
            f.WriteLine(TabString("Height", r.Height))
            f.WriteLine(TabString("Split1", Form_Main.SplitContainer1.SplitterDistance))
            f.WriteLine(TabString("Split2", Form_Main.SplitContainer2.SplitterDistance))
            '
            f.WriteLine("")
            f.WriteLine(" Editor options")
            f.WriteLine("======================================================================")
            f.WriteLine(TabString("Font", Form_Options.ComboBox_Font.Text))
            f.WriteLine(TabString("TabSize", Form_Options.ComboBox_TabSize.Text))
            f.WriteLine(TabString("Color_NormalText", Color_ToHtmlString(Form_Options.Label_NormalText.BackColor)))
            f.WriteLine(TabString("Color_QuotedText", Color_ToHtmlString(Form_Options.Label_QuotedText.BackColor)))
            f.WriteLine(TabString("Color_Keywords", Color_ToHtmlString(Form_Options.Label_Keywords.BackColor)))
            f.WriteLine(TabString("Color_Comments", Color_ToHtmlString(Form_Options.Label_Comments.BackColor)))
            f.WriteLine(TabString("Color_Errors", Color_ToHtmlString(Form_Options.Label_Errors.BackColor)))
            f.WriteLine(TabString("Color_Warnings", Color_ToHtmlString(Form_Options.Label_Warnings.BackColor)))
            f.WriteLine(TabString("Color_TextBack", Color_ToHtmlString(Form_Options.Label_TextBack.BackColor)))
            f.WriteLine(TabString("Color_EditorBack", Color_ToHtmlString(Form_Options.Label_EditorBack.BackColor)))
            f.WriteLine(TabString("Zoom", Form_Options.TrackBar1.Value))
            f.WriteLine(TabString("FileName", Editor_FileName))
            '
            f.WriteLine("")
            f.WriteLine(" Compiler options")
            f.WriteLine("======================================================================")
            f.WriteLine(TabString("Language", Form_Options.ComboBox_Language.Text))
            f.WriteLine(TabString("WarningLevel", Form_Options.ComboBox_WarningLevel.Text))
            f.WriteLine(TabString("OptionHideHeader", Form_Options.CheckBox_HideHeader.Checked))
            f.WriteLine(TabString("OptionExplicit", Form_Options.CheckBox_OptionExplicit.Checked))
            f.WriteLine(TabString("OptionStrict", Form_Options.CheckBox_OptionStrict.Checked))
            f.WriteLine(TabString("OptionOptimize", Form_Options.CheckBox_OptionOptimize.Checked))
            f.WriteLine(TabString("AddDebugInfo", Form_Options.CheckBox_AddDebugInfo.Checked))
            f.WriteLine(TabString("UseConsole", Form_Options.CheckBox_UseConsole.Checked))
            '
            f.WriteLine("")
            f.WriteLine(" Color dialog - custom colors")
            f.WriteLine("======================================================================")
            '
            For i As Int32 = 0 To CustomColors.Count - 1
                f.WriteLine("CustomColor", TAB(22), Color_IntToHtmlString(CustomColors(i)))
            Next
            f.WriteLine("")
            '
            f.Close()
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try
    End Sub


    Friend Sub LoadUserOptions()
        EventsAreEnabled = False
        ' ------------------------------------------------------------------------------- set defauts
        Form_Options.SetDefaultOptions()
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        ReDim CustomColors(-1)
        Dim l As String
        Dim iniFileName As String = PlatformAdjustedFileName(Application.StartupPath & "\" & _
                                                             AssemblyName() & "_UserOptions.txt")
        If My.Computer.FileSystem.FileExists(iniFileName) Then
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)
            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    ' ------------------------------------------------- main form
                    Case "Top" : Form_Main.Top = ExtractInteger(l, 0, 10000)
                    Case "Left" : Form_Main.Left = ExtractInteger(l, 0, 10000)
                    Case "Width" : Form_Main.Width = ExtractInteger(l, 0, 10000)
                    Case "Height" : Form_Main.Height = ExtractInteger(l, 0, 10000)
                    Case "Split1" : Form_Main.SplitContainer1.SplitterDistance = ExtractInteger(l, 0, 10000)
                    Case "Split2" : Form_Main.SplitContainer2.SplitterDistance = ExtractInteger(l, 0, 10000)
                        '
                        ' --------------------------------------------- editor options
                    Case "Font" : Combo_SetIndex_FromString(Form_Options.ComboBox_Font, l)
                    Case "TabSize" : Combo_SetIndex_FromString(Form_Options.ComboBox_TabSize, l)
                    Case "Color_NormalText" : Form_Options.Label_NormalText.BackColor = Color_FromHtmlString(l)
                    Case "Color_QuotedText" : Form_Options.Label_QuotedText.BackColor = Color_FromHtmlString(l)
                    Case "Color_Keywords" : Form_Options.Label_Keywords.BackColor = Color_FromHtmlString(l)
                    Case "Color_Comments" : Form_Options.Label_Comments.BackColor = Color_FromHtmlString(l)
                    Case "Color_Errors" : Form_Options.Label_Errors.BackColor = Color_FromHtmlString(l)
                    Case "Color_Warnings" : Form_Options.Label_Warnings.BackColor = Color_FromHtmlString(l)
                    Case "Color_TextBack" : Form_Options.Label_TextBack.BackColor = Color_FromHtmlString(l)
                    Case "Color_EditorBack" : Form_Options.Label_EditorBack.BackColor = Color_FromHtmlString(l)
                    Case "Zoom" : Form_Options.TrackBar1.Value = Val_Int(l)
                    Case "FileName" : Editor_FileName = l
                        '
                        ' --------------------------------------------- compiler options
                    Case "Language" : Combo_SetIndex_FromString(Form_Options.ComboBox_Language, l)
                    Case "WarningLevel" : Combo_SetIndex_FromString(Form_Options.ComboBox_WarningLevel, l)
                    Case "OptionHideHeader" : Form_Options.CheckBox_HideHeader.Checked = CBool(l)
                    Case "OptionExplicit" : Form_Options.CheckBox_OptionExplicit.Checked = CBool(l)
                    Case "OptionStrict" : Form_Options.CheckBox_OptionStrict.Checked = CBool(l)
                    Case "OptionOptimize" : Form_Options.CheckBox_OptionOptimize.Checked = CBool(l)
                    Case "AddDebugInfo" : Form_Options.CheckBox_AddDebugInfo.Checked = CBool(l)
                    Case "UseConsole" : Form_Options.CheckBox_UseConsole.Checked = CBool(l)
                        '
                        ' --------------------------------------------- custom colors
                    Case "CustomColor"
                        Dim index As Int32 = CustomColors.Count
                        ReDim Preserve CustomColors(index)
                        'CustomColors(index) = IntVal(l)
                        CustomColors(index) = Color_IntFromHtmlString(l)
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form_Main)
        Form_Options.SendAllOptions_ToEditor()
        EventsAreEnabled = True
    End Sub


End Module
