//================================================================================================
//  THEREMINO SCRIPT
//================================================================================================
// Theremino Script semplifica al massimo la gestione di semplici operazioni
// del tipo "accendo un led", "attendo un secondo", "spengo il led", "ripeto tre volte"
// Le complessita' di DotNet vengono nascoste per quanto possibile.
//------------------------------------------------------------------------------------------------
// L'approccio semplificato e' valido solo per semplici programmi di TEST.
// Nel caso che il progetto si allunghi oltre qualche pagina si consiglia di usare Visual Studio
//  di Microsoft che e' il fratello maggiore di questo programma.
//------------------------------------------------------------------------------------------------
// La versione Express di Visual Studio e' gratuita e funziona molto meglio di questo rozzo SCRIPT
//------------------------------------------------------------------------------------------------
// E' bene sapere che ad ogni RUN il programma viene salvato automaticamente.
// Se qualcosa va storto, salvare, chiudere e riaprire.
//================================================================================================


//================================================================================================
//  Example 3
//=================================================================================================
// Questo esempio mostra come accedere agli slot del sistema anche con il linguaggio CSharp 
// Con il sistema theremino si usa normalmente il VbNet ma per chi è abituato ad Arduino
// il linguaggio CSharp é molto più comodo (quasi identico al C di Arduino)
//=================================================================================================


//=================================================================================================
// Variabili e costanti
// Qui si dichiarano le variabili e le costanti che deveono essere in comune a tutte le funzioni
//=================================================================================================
//const int K1 = 12345;
//static bool BooleanVar1 = false;
//static int IntVar1 = 12345;
//static float FloatVar1 = 123.45F; // la "F" serve per impostare un valore letterale di tipo Float


//-------------------------------------------------------------------------------------------------
// Nella funzione "Main_Thread" si puo' eseguire ogni genere di operazione
// Si possono fare PAUSE lunghe a piacere, cicli LOOP, FOR, WHILE etc...
// Però non è bene aggiornare i controlli della finestra in questa funzione, 
// se necessario si impostano variabili comuni che verranno poi visualizzate dalla Timed_Update 
//-------------------------------------------------------------------------------------------------
// Se si arriva alla fine della funzione allora la funzione ricomincia immediatamente da capo
//-------------------------------------------------------------------------------------------------
static void Main_Thread()
{
	// Esempio che divide il valore dello Slot1 in quattro parti e imposta gli slot 2, 3, 4 e 5
	// Utilizzare Theremino SlotViewer per vedere i risultati
	// Non è necessario collegare un Master, molte cose si fanno anche senza hardware,
	// si possono pilotare applicazioni che suonano o leggere valori da una WebCam
	// L'hardware serve solo qando si deve uscire o entrare dal PC attraverso la USB
	
	float n = ReadSlot(1);
	
	switch ((int)n / 250)
	{
		case 0:
			WriteSlot(2, 1000);	
			WriteSlot(3, 0);
			WriteSlot(4, 0);
			WriteSlot(5, 0);
			break;
		case 1:
			WriteSlot(2, 0);	
			WriteSlot(3, 1000);
			WriteSlot(4, 0);
			WriteSlot(5, 0);
			break;
		case 2:
			WriteSlot(2, 0);	
			WriteSlot(3, 0);
			WriteSlot(4, 1000);
			WriteSlot(5, 0);
			break;
		case 3:
			WriteSlot(2, 0);	
			WriteSlot(3, 0);
			WriteSlot(4, 0);
			WriteSlot(5, 1000);
			break;
	}
}

// ------------------------------------------------------------------------------------------------
//  Questa funzione viene chiamata 20 volte al secondo
//  L'aggiornamento dei controlli che vanno rinfrescati regolarmente deve avvenire qui
//  Ma niente PAUSE e niente LOOP, solo esecuzioni rapide
//  Aggiornare i valori (ad esempio i "Text" delle textbox) e poi uscire subito dalla funzione
// ------------------------------------------------------------------------------------------------
static void Timed_Update()
{
	// Notare che l'aggiornamento delle label è stato aggiunto qui al posto che in Main_Thread
	// Qui avviene 20 volte al secondo (la velocità giusta per le comunicazioni con l'utente)
	// Invece nella Main_Thread avverrebbe 1000 volte al secondo e caricherebbe molto la CPU
	// (oppure avverrebbe con molto ritardo se nella Main_Thread ci fossero pause o loops)
	// Non ha importanza che così facendo si devono rileggere gli slot, sono velocissimi (<1uS)
	Label1.Text = "Slot 1 = " + ReadSlot(1).ToString(); 
	Label2.Text = "Slot 2 = " + ReadSlot(2).ToString(); 
	Label3.Text = "Slot 3 = " + ReadSlot(3).ToString(); 
	Label4.Text = "Slot 4 = " + ReadSlot(4).ToString(); 
	Label5.Text = "Slot 5 = " + ReadSlot(5).ToString(); 
}

// ------------------------------------------------------------------------------------------------
//  Le seguenti funzioni sono "Eventi" - Vengono chiamate al verificarsi degli eventi relativi
//  Abilitando i beep si può "sentire" quando gli eventi accadono
// ------------------------------------------------------------------------------------------------
static void Window_Opening()
{
	//Beep();
	Label6.Text = "Window opened";
}

static void Window_Closing()
{
	//Beep();
}

static void TrackBar1_ValueChanged(Object myObject, EventArgs myEventArgs)
{
	//Beep();
	Label6.Text = "TrackBar1 value changed - Value = " + TrackBar1.Value.ToString(); 
	WriteSlot(1, TrackBar1.Value);

}

static void TrackBar2_ValueChanged(Object myObject, EventArgs myEventArgs)
{
	//Beep();
	Label6.Text = "TrackBar2 value changed - updating Colors";
	int n255 = (TrackBar2.Value * 255 / 1000);
	Color c = Color.FromArgb(n255, 255, 255-n255);
	Label6.BackColor = c;
}

static void Button1_Click(Object myObject, EventArgs myEventArgs)
{
	Beep();
	Label6.Text = "Button 1 pressed";
	TrackBar2.Value = 800;
}

static void Button2_Click(Object myObject, EventArgs myEventArgs)
{
	Beep();
	Label6.Text = "Button 2 pressed";
	TrackBar2.Value = 300;
}

static void Button3_Click(Object myObject, EventArgs myEventArgs)
{
	Beep();
	Label6.Text = "Button 3 pressed";
	Button1.Text = "NOOOOO !!!!";
	Button2.Text = "AAAAAA !!!";	
	Button3.Text = "PRESSED !!!";
	for (int i = 0; i <= 1000; i+=50)
	{ 
		TrackBar2.Value = i;
		DelayMillisec(100);
		Form1.Refresh();
		Beep();
	}
	Button1.Text = "Button 1";
	Button2.Text = "Button 2";	
	Button3.Text = "Do not press !";
}

// ------------------------------------------------------------------------------------------------
// Componenti della interfaccia utente
// Qui si possono aggiungere controlli e modificare le loro proprietà
// Per aggiungere un nuovo controllo copiarne uno esistente
// Ricordarsi di cambiare nome ai nuovi controlli (Label2, Label3 etc..) 
// Ricordarsi di cambiare posizione ai nuovi controlli (altrimenti sono sopra ai precedenti)
// Per alcuni controlli è necessario avere un evento per cui si deve anche aggiungere la funzione
// (e incrementarne il nome dell'evento: TrackBar2_ValueChanged, TrackBar3_ValueChanged..)
// ------------------------------------------------------------------------------------------------
static Label Label1 = new Label();
static Label Label2 = new Label();
static Label Label3 = new Label();
static Label Label4 = new Label();
static Label Label5 = new Label();
static Label Label6 = new Label();
static TrackBar TrackBar1 = new TrackBar();
static TrackBar TrackBar2 = new TrackBar();
static Button Button1 = new Button();
static Button Button2 = new Button();
static Button Button3 = new Button();

static void InitializeComponents()
{
	// ------------------------------------------------------------------ Form (cambiare il titolo e la posizione)
	Form1.Text = "Theremino Script - Example 3";
	Form1.ClientSize = new Size(640, 320);
	Form1.MinimumSize = Form1.Size;
	Form1.StartPosition = FormStartPosition.CenterScreen;
	// ------------------------------------------------------------------ Label 1
	Label1.BorderStyle = BorderStyle.Fixed3D;
	Label1.Location = new Point(10, 10);
	Label1.Font = new Font("Arial", 11);
	Label1.Size = new Size(100, 20);
	Form1.Controls.Add(Label1);
	// ------------------------------------------------------------------ Label 2
	Label2.BorderStyle = BorderStyle.Fixed3D;
	Label2.Location = new Point(120, 10);
	Label2.Font = new Font("Arial", 11);
	Label2.Size = new Size(100, 20);
	Form1.Controls.Add(Label2);
	// ------------------------------------------------------------------ Label 3
	Label3.BorderStyle = BorderStyle.Fixed3D;
	Label3.Location = new Point(230, 10);
	Label3.Font = new Font("Arial", 11);
	Label3.Size = new Size(100, 20);
	Form1.Controls.Add(Label3);
	// ------------------------------------------------------------------ Label 4
	Label4.BorderStyle = BorderStyle.Fixed3D;
	Label4.Location = new Point(340, 10);
	Label4.Font = new Font("Arial", 11);
	Label4.Size = new Size(100, 20);
	Form1.Controls.Add(Label4);
	// ------------------------------------------------------------------ Label 5
	Label5.BorderStyle = BorderStyle.Fixed3D;
	Label5.Location = new Point(450, 10);
	Label5.Font = new Font("Arial", 11);
	Label5.Size = new Size(100, 20);
	Label5.Anchor = AnchorStyles.Top | AnchorStyles.Left | AnchorStyles.Right;
	Form1.Controls.Add(Label5);
	// ------------------------------------------------------------------ Label 6
	Label6.BorderStyle = BorderStyle.Fixed3D;
	Label6.Location = new Point(10, Form1.ClientSize.Height - 30);
	Label6.Font = new Font("Arial", 11);
	Label6.Size = new Size(Form1.ClientSize.Width - 30, 21);
	// Questo AnchorStyle aggancia la label in basso e la stira con la finestra
	Label6.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right;
	Form1.Controls.Add(Label6);
	// ------------------------------------------------------------------ TrackBar 1
	TrackBar1.Location = new Point(10, 60);
	TrackBar1.Size = new Size(45, 200);
	TrackBar1.Orientation = Orientation.Vertical;
	TrackBar1.TickStyle = TickStyle.Both;
	TrackBar1.Maximum = 1000;
	TrackBar1.TickFrequency = 100;
	TrackBar1.LargeChange = 10;
	Form1.Controls.Add(TrackBar1);
	TrackBar1.ValueChanged += new EventHandler(TrackBar1_ValueChanged);
	// ------------------------------------------------------------------ TrackBar 2
	TrackBar2.Location = new Point(80, 60);
	TrackBar2.Size = new Size(45, 200);
	TrackBar2.Orientation = Orientation.Vertical;
	TrackBar2.TickStyle = TickStyle.Both;
	TrackBar2.Maximum = 1000;
	TrackBar2.TickFrequency = 100;
	TrackBar2.LargeChange = 10;
	Form1.Controls.Add(TrackBar2);
	TrackBar2.ValueChanged += new EventHandler(TrackBar2_ValueChanged);
	// ------------------------------------------------------------------ Button 1
	Button1.Text = "Button 1";
	Button1.Location = new Point(200, 120);
	Button1.Size = new Size(180, 22);
	Form1.Controls.Add(Button1);
	Button1.Click += new EventHandler(Button1_Click);
	// ------------------------------------------------------------------ Button 2
	Button2.Text = "Button 2";
	Button2.Location = new Point(200, 160);
	Button2.Size = new Size(180, 22);
	Form1.Controls.Add(Button2);
	Button2.Click += new EventHandler(Button2_Click);
	// ------------------------------------------------------------------ Button 3
	Button3.Text = "Do not press";
	Button3.Location = new Point(200, 200);
	Button3.Size = new Size(180, 22);
	Form1.Controls.Add(Button3);
	Button3.Click += new EventHandler(Button3_Click);
}

// ------------------------------------------------------------------------------------------------
//  Funzioni di utilità semplificate
// ------------------------------------------------------------------------------------------------
static void DelayMillisec(Int32 mS)
{
	System.Threading.Thread.Sleep(mS);
}
static void Beep()
{
	System.Media.SystemSounds.Beep.Play();
}


// ===============================================================================================
//  LA GESTIONE SEMPLIFICATA FINISCE QUI
//  MODIFICARE LA PARTE SEGUENTE CON ATTENZIONE
// ===============================================================================================
static Form Form1;
static void Main(String[] pars)
{
	if (Form1 != null) return;
	Form1 = new Form();
	Form1.Closing += new CancelEventHandler(Form1_Closing);
	Application.EnableVisualStyles();
	Form1.SuspendLayout();
	InitializeComponents();
	Form1.ResumeLayout();
	Form1.PerformLayout();
	StartMainTimer();
	StartMainThread();
	Window_Opening();
	Form1.Show();
	while(Form1.Visible)
	{
		Application.DoEvents();
		System.Threading.Thread.Sleep(1);
	}
}
static void Form1_Closing(Object myObject, EventArgs myEventArgs)
{
	MainTimer.Stop();
	MainThread.Abort();
	Window_Closing();
}
// ---------------------------------------------------------------------- Main Thread
static System.Threading.Thread MainThread;
static void StartMainThread()
{
	MainThread = new System.Threading.Thread(ExecuteThread);
	MainThread.IsBackground = true;
	MainThread.Priority = System.Threading.ThreadPriority.AboveNormal;
	MainThread.Start();
}
static void ExecuteThread()
{
	while (true)
	{
		Main_Thread();
		System.Threading.Thread.Sleep(1);
	}
}
// ---------------------------------------------------------------------- Main Timer
static Timer MainTimer;
static void StartMainTimer()
{
	MainTimer = new Timer();
	MainTimer.Tick += new EventHandler(Timer1_Tick);
	MainTimer.Interval = 50;
	MainTimer.Start();
}
static void Timer1_Tick(Object myObject, EventArgs myEventArgs)
{
	Timed_Update();
}


