' ================================================================================================
' - ENGLISH - TEMPERATURE TRIGGER - MULTIPLE SLOTS - DELTA TEMPERATURE
' ================================================================================================
'  This script reads a specified number of temperature sensors
'  If a sensor temperature increment is more then value "DeltaTempTrigger"
'   a high signal (1000) is emitted for 100 mS on the corresponding output slot
' ------------------------------------------------------------------------------------------------

' ================================================================================================
' - ITALIANO - TEST DI TEMPERATURA A SOGLIA - PER SLOT MULTIPLI - A VARIAZIONE DI TEMPERATURA
' ================================================================================================
'  Questo script legge un certo numero di sensori di temperatura
'  Se l'incremento di temperatura di un sensore supera il valore "DeltaTempTrigger"
'   viene emesso un segnale alto (1000) per 100 mS, sul corrisponedente slot di uscita
' ------------------------------------------------------------------------------------------------

Const DeltaTempTrigger As Single = 20
Const FirstInputSlot As Int32 = 1
Const FirstOutputSlot As Int32 = 11
Const NumSlots As Int32 = 5

Dim OldTemp(NumSlots -1) As Single
Dim NewTemp(NumSlots -1) As Single
Dim Ready(NumSlots -1) As Boolean

Dim ci As Globalization.Cultureinfo = Globalization.Cultureinfo.InvariantCulture
Dim sw As StopWatch = New StopWatch

Sub Main(ByVal s() As String)
	If Form1 IsNot Nothing Then Return
	Form1 = New Form
	InitializeComponents
	MemoryMappedFile_Init
	sw.start
	While Form1.Visible
		Application.DoEvents
		Threading.Thread.Sleep(10)
	End While
End Sub

Sub Form1_Closing(sender As Object, e As CancelEventArgs) Handles Form1.Closing
	Timer1.Stop
End Sub

Sub Timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles Timer1.Tick
	Dim Temp As Single
	Dim DeltaTemp As Single
	Dim s As String = "Ch.    Temp       DeltaTemp" + vbcrlf
	For i As Int32 = 0 To NumSlots -1
		' ----------------------------------------------------- calc the temperature
		Temp = ReadSlot(FirstInputSlot + i) / 3
		' ----------------------------------------------------- smooth the temperature
		SmoothValue_Pow_Adaptive(NewTemp(i), Temp, 10)
		' ----------------------------------------------------- calc the delta temperature
		DeltaTemp = (NewTemp(i) - OldTemp(i)) * 10000
		' ----------------------------------------------------- test trigger
		If DeltaTemp >= DeltaTempTrigger Then
			' ------------------------------------------------- test if Ready
			If Ready(i) Then
				WriteSlot(FirstOutputSlot + i, 1000)
				sw.reset
				sw.start
				Ready(i) = False
			End If
		End If
		' ----------------------------------------------------- rearm Ready when temperature decreases
		If DeltaTemp < 0 Then
			Ready(i) = True	
		End If
		' ----------------------------------------------------- reset output after 100 mS
		If sw.ElapsedMilliseconds > 100 Then
			WriteSlot(FirstOutputSlot + i, 0)
		End If
		' ----------------------------------------------------- update old values
		OldTemp(i) = NewTemp(i)
		' ----------------------------------------------------- show channel data
		s += (i +1).ToString("00      ") + NewTemp(i).Tostring("0.000       ", ci) + DeltaTemp.Tostring("0", ci) + vbcrlf
	Next
	Label1.Text = s
End Sub


' ================================================================================================
'   An adaptive, HiQuality, smoothing function
' ================================================================================================
Private Sub SmoothValue_Pow_Adaptive(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
	Dim delta As Single = new_value - value
	Dim delta_sgn As Single = Math.Sign(delta)
	Dim delta_abs As Single = Math.Abs(delta)
	Dim delta_pow As Single = CSng(delta_abs ^ 2) * speed
	If value <> 0 Then delta_pow /= abs(value)
	If delta_pow >= delta_abs Then
		value = new_value
	Else
		value += delta_sgn * delta_pow
	End If
End Sub

' ================================================================================================
'   User Interface Components
' ================================================================================================
WithEvents Form1 As Form
WithEvents Timer1 As Timer = New Timer
Dim Label1 As Label = New Label

Sub InitializeComponents()
	Application.EnableVisualStyles
	form1.SuspendLayout
	' ------------------------------------------------------------------ Form
	Form1.Text = "Theremino Script - Delta Temp Trigger"
	Form1.FormBorderStyle = FormBorderStyle.Fixed3D
	Form1.MaximizeBox = False
	Form1.ClientSize = New System.Drawing.Size(340, NumSlots * 26 + 26)
	Form1.MinimumSize = Form1.Size
	Form1.StartPosition = FormStartPosition.CenterScreen
	' ------------------------------------------------------------------ Label 1
	'Label1.BorderStyle = BorderStyle.Fixed3D
	Label1.Location = New Point(0, 0)
	Label1.Size = Form1.ClientSize
	Label1.BackColor = Color.DarkBlue
	Label1.ForeColor = Color.Gold
	Label1.Font = New Font("Tahoma", 16, FontStyle.Bold)
	Label1.TextAlign = ContentAlignment.TopLeft
	Label1.Anchor = AnchorStyles.Top Or AnchorStyles.Left Or _
					AnchorStyles.Right Or AnchorStyles.Bottom
	Form1.Controls.Add(Label1)
	' ------------------------------------------------------------------ Timer 1
	Timer1.Interval = 15
	Timer1.Start
	' ------------------------------------------------------------------ 
	Form1.ResumeLayout
	Form1.PerformLayout()
	Form1.Show
End Sub
